@@ -1,7 +1,14 @@
 """A file containing useful predicates."""
 
-from typing import List, Iterable, Callable, Any, Sequence, Union
-from .higher_order_functions import partial, flip
+from typing import List, Iterable, Callable, Any, Sequence, Union, Tuple, Iterator
+
+from .higher_order_functions import partial, flip, reduce
+from operator import lt, le, ge, gt
+
+
+def identity(x):
+    """Returns its argument as it is."""
+    return x
 
 
 def equals(a, b=None, *args):
@@ -118,15 +125,16 @@ def less_than(a, b=None, *args):
     elif not args:
         return a < b
     else:
-        # TODO ugly. refactor
-        if a < b:
-            for arg in args:
-                if a >= arg:
-                    return False
+
+        def f(boolean, el, args):
+            if not boolean:
+                return False
+            elif not args:
+                return boolean
             else:
-                return True
-        else:
-            return False
+                return f(lt(el, args[0]), args[0], args[1:])
+
+        return f(lt(a, b), b, args)
 
 
 def less_than_eq(a, b=None, *args):
@@ -164,14 +172,15 @@ def less_than_eq(a, b=None, *args):
         return a <= b
     else:
         # TODO ugly. refactor
-        if a <= b:
-            for arg in args:
-                if a > arg:
-                    return False
+        def f(boolean, el, args):
+            if not boolean:
+                return False
+            elif not args:
+                return boolean
             else:
-                return True
-        else:
-            return False
+                return f(le(el, args[0]), args[0], args[1:])
+
+        return f(le(a, b), b, args)
 
 
 def greater_than(a, b=None, *args):
@@ -209,14 +218,15 @@ def greater_than(a, b=None, *args):
         return a > b
     else:
         # TODO ugly. refactor
-        if a > b:
-            for arg in args:
-                if a <= arg:
-                    return False
+        def f(boolean, el, args):
+            if not boolean:
+                return False
+            elif not args:
+                return boolean
             else:
-                return True
-        else:
-            return False
+                return f(gt(el, args[0]), args[0], args[1:])
+
+        return f(gt(a, b), b, args)
 
 
 def greater_than_eq(a, b=None, *args):
@@ -254,14 +264,15 @@ def greater_than_eq(a, b=None, *args):
         return a >= b
     else:
         # TODO ugly. refactor
-        if a >= b:
-            for arg in args:
-                if a < arg:
-                    return False
+        def f(boolean, el, args):
+            if not boolean:
+                return False
+            elif not args:
+                return boolean
             else:
-                return True
-        else:
-            return False
+                return f(ge(el, args[0]), args[0], args[1:])
+
+        return f(ge(a, b), b, args)
 
 
 def complement(
@@ -381,7 +392,7 @@ def is_numeric(entity: Any) -> bool:
 
 
 def is_atom(entity: Any) -> bool:
-    """Everything that is NOT an iterable( except strings ) Are considered atoms.
+    """Everything that is NOT an iterable(except strings) are considered atoms.
 
     >>> is_atom("plain string")
         True
@@ -416,11 +427,13 @@ def is_empty(collection: Iterable) -> bool:
 
 
 def is_nested(collection: Iterable) -> bool:
-    """returns true if a collection is nested
-
+    """returns true if a collection is nested.
     Added in version: 0.1.0
     """
-    return any(map(complement(is_atom), collection))
+    if isinstance(collection, dict):
+        return any(map(lambda x: isinstance(x, dict), collection.values()))
+    else:
+        return any(map(complement(is_atom), collection))
 
 
 def all_predicates(*predicates: Callable[[Any], bool]) -> Callable[[Any], bool]:
