@@ -11,12 +11,12 @@ from typing import (
     Callable,
     Iterator,
 )
+
 import sys
 
 
 def iter_(iterable: Iterable) -> Iterator:
     """Returns appropriate iterator for the given iterable.
-    If iterable is already an iterator, it is returned as is.
     This is mainly created because python's ``iter``
     returns an iterable of keys instead of keys and values for ``dict``.
 
@@ -31,16 +31,11 @@ def iter_(iterable: Iterable) -> Iterator:
         # I want a tuple of key-value pairs
         return iter(iterable.items())
 
-    elif not isinstance(iterable, Iterator):
-        return iter(iterable)
-
-    else:  # if iterable is already an iterator
-        return iterable
+    return iter(iterable)
 
 
 def reversed_(iterable: Iterable) -> Iterator:
     """Returns appropriate reversed iterator for the given iterable.
-    If iterable is already an iterator, it is returned as is.
     This is mainly created because python's ``reversed``
     returns an iterable of keys instead of keys and values for ``dict``.
 
@@ -49,21 +44,26 @@ def reversed_(iterable: Iterable) -> Iterator:
 
     Added in version: 0.1.0
     """
+    # TODO drop support for python < 3.8 when support ends
     if isinstance(iterable, dict):
         # since iter(dict) returns a tuple of keys.
         # I want a tuple of key-value pairs
+
         if (
             sys.version_info[1] < 8
         ):  # since reversed for dicts was available only in 3.8
             return reversed([(k, v) for k, v in iterable.items()])
-        else:
+        else:  # pragma:no cover
             return reversed(iterable.items())
 
     elif not isinstance(iterable, Iterator):
         return reversed(iterable)
 
-    else:  # if iterable is already an iterator
-        return iterable
+    else:  # if iterable is already a reversed iterator
+        try:
+            return reversed(iterable)
+        except TypeError:
+            return reversed(tuple(iterable))
 
 
 def first(iterable: Iterable[Any]) -> Union[Any, None]:
@@ -110,14 +110,11 @@ def last(iterable: Iterable[Any]) -> Union[Any, None]:
 
     Added in version: 0.1.0
     """
-    try:
-        # using a deque is an efficient way to get the last element
-        dq = deque(iter_(iterable), maxlen=1)
-
-        return dq.pop()
-
-    except IndexError:  # If iterable is empty, dq is empty
+    if not iterable:
         return None
+    else:
+        it = iter_(iterable)
+        return list(it)[-1]
 
 
 def rest(iterable: Iterable) -> Iterator:
@@ -345,3 +342,34 @@ def split_with(predicate: Callable, iterable: Iterable) -> Tuple[Tuple, Tuple]:
     # consider implementing with reduce
     # since we are iterating through iterable twice.
     return (take_while(predicate, iterable), drop_while(predicate, iterable))
+
+
+def count(iterable: Iterable) -> int:
+    """
+    counts the number of elements in the iterable, works with map objects, filter objets, and iterators.
+    ``count`` will consume iterators, use ``count_`` if you want access to the iterators.
+    Added in version: 0.1.2
+
+    >>> count(iter([1,2,3]))
+    3
+    """
+    if hasattr(iterable, "__len__"):
+        return len(iterable)
+    else:
+        return len(list(iterable))
+
+
+def count_(iterable: Iterable) -> Tuple[int, Iterable]:
+    """
+    returns a tuple of the number of elements in the iterable and the iterable itself.
+    This can be used if you wish to find the length of iterators and want to consume the iterators
+    later on.
+    Added in version: 0.1.2
+    >>> count(iter([1,2,3]))
+    (3,[1,2,3])
+    """
+    if hasattr(iterable, "__len__"):
+        return (len(iterable), iterable)
+    else:
+        l = list(iterable)
+        return (len(l), l)
